
/*
ESP32-C3S 賣場連結 : https://www.ruten.com.tw/item/show?22119396842939
設定說明 : https://youyouyou.pixnet.net/blog/post/121068678-esp32-c3f-%e5%96%ae%e6%a0%b8%e4%bd%8e%e5%83%b9%e6%94%af%e6%8f%b4ble5%e6%a6%82%e8%a7%80
開發板:"ESP32C3 Dev Module"
Flash Size: "2MB (16Mb)"
Partition Scheme: "Default 4MB with spiffs (1.2MB APP/1.5MB SPIFFS)"
打開串列埠視窗會造成無法動作，須改為 ESP32 Dev Module 才可開啟，但又改程式時必須記住上述參數要調回來
 */
#include <stdio.h>
#include <PubSubClient.h>
#include <WiFi.h>

const char* ssid = "mywifissid";
const char* pass =  "mywifipassword";
const char* mqttServer = "icdt.com.tw";
const int mqttPort = 1883;

const char* mqttUser = "aiqwebhmi";
const char* mqttPassword = "12345678";
//MqttSubscribe 開頭必須與 mqttUser 相同
const char* MqttSubscribe = "aiqwebhmi/#";
//clientId 開頭必須與 mqttUser 相同
String clientId = String(mqttUser);
bool Wifi_Ready = false;


WiFiClient espClient;
PubSubClient client(espClient);

// setting PWM properties
const int freq = 5000;
const int ledRChannel = 0;
const int ledGChannel = 1;
const int ledBChannel = 2;
const int ledWChannel = 3;
const int resolution = 8;
const int Powerpin = 19; //內建電源指示燈
const int RGBRedpin = 3; //內建RGB紅色LED燈
const int RGBGreenpin = 4; //內建RGB綠色LED燈
const int RGBBluepin = 5; //內建RGB藍色LED燈


String IpAddress2String(const IPAddress& ipAddress){
  return String(ipAddress[0]) + String(".") +
         String(ipAddress[1]) + String(".") +
         String(ipAddress[2]) + String(".") +
         String(ipAddress[3]);
}

void printWifiStatus() {
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());
  IPAddress ip = WiFi.localIP();
  clientId = clientId + IpAddress2String(ip);
  Serial.print("IP Address: ");
  Serial.println(ip);
}

//WIFI 連線
void setupWiFi_ST() {
  // setting up Station AP
  WiFi.begin(ssid, pass);
  // Wait for connect to AP
  Serial.print("[Connecting]");
  Serial.print(ssid);
  int tries = 0;
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
    tries++;
    if (tries > 10) {
      Serial.println("Wifi Fault!");
      return;
    }
  }
  Wifi_Ready = true;
  Serial.println();
  printWifiStatus();
}

//MQTT 檢查連線狀態
void reconnect() {
  while (!client.connected()) {
    Serial.println("Connecting to MQTT...");
    if (client.connect(clientId.c_str(),mqttUser,mqttPassword)) {
      client.subscribe(MqttSubscribe);
      Serial.println("connected");
    } else {
      Serial.print("failed with state ");
      Serial.print(client.state());
      delay(5000);
    }
  }
}

//MQTT 收到發布資料
void callback(char* topic, byte* payload, unsigned int length){
  Serial.print("Message arrived in topic: ");
  String sTopic = topic;
  sTopic = sTopic.substring(sTopic.lastIndexOf("/") + 1);
  payload[length] = 0;
  float v = atof((char *)payload);
  
  if(sTopic == "LEDW")
  {
    ledcWrite(ledWChannel, v);
  } 
  else if(sTopic == "LEDR")
  {
    ledcWrite(ledRChannel, v);
  }
  else if(sTopic == "LEDG")
  {
    ledcWrite(ledGChannel, v);
  }
  else if(sTopic == "LEDB")
  {
    ledcWrite(ledBChannel, v);
  }
  Serial.println(sTopic + "=" + v);
}

void setup() {
  Serial.begin(115200);
  pinMode(Powerpin, OUTPUT);

  while (!Wifi_Ready)
  {
    digitalWrite(Powerpin, HIGH);
    setupWiFi_ST();
    digitalWrite(Powerpin, LOW);
    delay(1000);
  }
  //設定 PWM 輸出
  ledcSetup(ledRChannel, freq, resolution);
  ledcSetup(ledGChannel, freq, resolution);
  ledcSetup(ledBChannel, freq, resolution);
  ledcSetup(ledWChannel, freq, resolution);
  ledcAttachPin(RGBRedpin, ledRChannel);
  ledcAttachPin(RGBGreenpin, ledGChannel);
  ledcAttachPin(RGBBluepin, ledBChannel);
  ledcAttachPin(Powerpin, ledWChannel);
  //設定 MQTT Client
  client.setServer(mqttServer, mqttPort);
  client.setCallback(callback);
}

bool initRequest = false;
void loop() {
  if (!client.connected()) {
    reconnect();
  }
  if(!initRequest)
  {
    //剛啟動要求重送目前狀態目前
    initRequest = true; 
    char topic[40];
    char buf[2];
    sprintf (buf, "?");
    sprintf (topic,"%s/LEDR",mqttUser);
    client.publish(topic, buf);
    sprintf (topic,"%s/LEDG",mqttUser);
    client.publish(topic, buf);
    sprintf (topic,"%s/LEDB",mqttUser);
    client.publish(topic, buf);
  }
  client.loop();
}
